@@ -3,7 +3,7 @@
  *  Regression tests for the exhashmap<> container. */
 
 /*
- *  GiNaC Copyright (C) 1999-2005 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2009 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -20,7 +20,11 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include "exams.h"
+#include "ginac.h"
+using namespace GiNaC;
+
+#include <iostream>
+using namespace std;
 
 unsigned exam_hashmap()
 {
@@ -28,7 +32,6 @@ unsigned exam_hashmap()
 	unsigned N = 100;
 
 	cout << "examining hash maps" << flush;
-	clog << "----------hash maps:" << endl;
 
 	// Create empty container
 	exhashmap<unsigned> M1;
@@ -207,7 +210,7 @@ unsigned exam_hashmap()
 	}
 	if (it->second != 0) {
 		clog << "find() returned an incorrect value" << endl;
-		+result;
+		++result;
 	}
 
 	M5.erase(it);
@@ -279,15 +282,12 @@ unsigned exam_hashmap()
 		clog << "count(4*x^y) returns " << n << " instead of 1" << endl;
 		++result;
 	}
-
 	cout << '.' << flush;
 
-	if (!result) {
-		cout << " passed " << endl;
-		clog << "(no output)" << endl;
-	} else {
-		cout << " failed " << endl;
-	}
-
 	return result;
 }
+
+int main(int argc, char** argv)
+{
+	return exam_hashmap();
+}
