@@ -4,7 +4,7 @@
  *  functions. */
 
 /*
- *  GiNaC Copyright (C) 1999-2008 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2009 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -21,11 +21,12 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include <iostream>
 #include "ginac.h"
-using namespace std;
 using namespace GiNaC;
 
+#include <iostream>
+using namespace std;
+
 /* Assorted tests on other transcendental functions. */
 static unsigned inifcns_consist_trans()
 {
@@ -215,6 +216,20 @@ static unsigned inifcns_consist_zeta()
 	return result;
 }
 
+static unsigned inifcns_consist_various()
+{
+	unsigned result = 0;
+	symbol n;
+	ex e;
+	
+	if ( binomial(n, 0) != 1 ) {
+		clog << "ERROR: binomial(n,0) != 1" << endl;		
+		++result;
+	}
+	
+	return result;
+}
+
 unsigned exam_inifcns()
 {
 	unsigned result = 0;
@@ -225,6 +240,7 @@ unsigned exam_inifcns()
 	result += inifcns_consist_gamma();  cout << '.' << flush;
 	result += inifcns_consist_psi();  cout << '.' << flush;
 	result += inifcns_consist_zeta();  cout << '.' << flush;
+	result += inifcns_consist_various();  cout << '.' << flush;
 	
 	return result;
 }
