@@ -3,7 +3,7 @@
  *  Rational function normalization test suite. */
 
 /*
- *  GiNaC Copyright (C) 1999-2007 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2009 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -20,7 +20,11 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include "exams.h"
+#include "ginac.h"
+using namespace GiNaC;
+
+#include <iostream>
+using namespace std;
 
 static symbol w("w"), x("x"), y("y"), z("z");
 
@@ -219,7 +223,6 @@ unsigned exam_normalization()
 	unsigned result = 0;
 	
 	cout << "examining rational function normalization" << flush;
-	clog << "----------rational function normalization:" << endl;
 	
 	result += exam_normal1(); cout << '.' << flush;
 	result += exam_normal2(); cout << '.' << flush;
@@ -227,12 +230,10 @@ unsigned exam_normalization()
 	result += exam_normal4(); cout << '.' << flush;
 	result += exam_content(); cout << '.' << flush;
 	
-	if (!result) {
-		cout << " passed " << endl;
-		clog << "(no output)" << endl;
-	} else {
-		cout << " failed " << endl;
-	}
-	
 	return result;
 }
+
+int main(int argc, char** argv)
+{
+	return exam_normalization();
+}
