@@ -3,7 +3,7 @@
  *  Implementation of GiNaC's clifford algebra (Dirac gamma) objects. */
 
 /*
- *  GiNaC Copyright (C) 1999-2008 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2009 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -20,8 +20,6 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include <stdexcept>
-
 #include "clifford.h"
 
 #include "ex.h"
@@ -40,14 +38,14 @@
 #include "archive.h"
 #include "utils.h"
 
+#include <stdexcept>
+
 namespace GiNaC {
 
 GINAC_IMPLEMENT_REGISTERED_CLASS_OPT(clifford, indexed,
   print_func<print_dflt>(&clifford::do_print_dflt).
   print_func<print_latex>(&clifford::do_print_latex))
 
-const tinfo_static_t clifford::return_type_tinfo_static[256] = {{}};
-
 GINAC_IMPLEMENT_REGISTERED_CLASS_OPT(diracone, tensor,
   print_func<print_dflt>(&diracone::do_print).
   print_func<print_latex>(&diracone::do_print_latex))
@@ -78,7 +76,6 @@ GINAC_IMPLEMENT_REGISTERED_CLASS_OPT(diracgammaR, tensor,
 
 clifford::clifford() : representation_label(0), metric(0), commutator_sign(-1)
 {
-	tinfo_key = &clifford::tinfo_static;
 }
 
 DEFAULT_CTOR(diracone)
@@ -97,7 +94,6 @@ DEFAULT_CTOR(diracgammaR)
  *  @see dirac_ONE */
 clifford::clifford(const ex & b, unsigned char rl) : inherited(b), representation_label(rl), metric(0), commutator_sign(-1)
 {
-	tinfo_key = &clifford::tinfo_static;
 }
 
 /** Construct object with one Lorentz index. This constructor is for internal
@@ -107,25 +103,28 @@ clifford::clifford(const ex & b, unsigned char rl) : inherited(b), representatio
 clifford::clifford(const ex & b, const ex & mu, const ex & metr, unsigned char rl, int comm_sign) : inherited(b, mu), representation_label(rl), metric(metr), commutator_sign(comm_sign)
 {
 	GINAC_ASSERT(is_a<varidx>(mu));
-	tinfo_key = &clifford::tinfo_static;
 }
 
 clifford::clifford(unsigned char rl, const ex & metr, int comm_sign, const exvector & v, bool discardable) : inherited(not_symmetric(), v, discardable), representation_label(rl), metric(metr), commutator_sign(comm_sign)
 {
-	tinfo_key = &clifford::tinfo_static;
 }
 
 clifford::clifford(unsigned char rl, const ex & metr, int comm_sign, std::auto_ptr<exvector> vp) : inherited(not_symmetric(), vp), representation_label(rl), metric(metr), commutator_sign(comm_sign)
 {
-	tinfo_key = &clifford::tinfo_static;
+}
+
+return_type_t clifford::return_type_tinfo() const
+{
+	return make_return_type_t<clifford>(representation_label);
 }
 
 //////////
 // archiving
 //////////
 
-clifford::clifford(const archive_node & n, lst & sym_lst) : inherited(n, sym_lst)
+void clifford::read_archive(const archive_node& n, lst& sym_lst)
 {
+	inherited::read_archive(n, sym_lst);
 	unsigned rl;
 	n.find_unsigned("label", rl);
 	representation_label = rl;
@@ -142,13 +141,12 @@ void clifford::archive(archive_node & n) const
 	n.add_unsigned("commutator_sign+1", commutator_sign+1);
 }
 
-DEFAULT_UNARCHIVE(clifford)
-DEFAULT_ARCHIVING(diracone)
-DEFAULT_ARCHIVING(cliffordunit)
-DEFAULT_ARCHIVING(diracgamma)
-DEFAULT_ARCHIVING(diracgamma5)
-DEFAULT_ARCHIVING(diracgammaL)
-DEFAULT_ARCHIVING(diracgammaR)
+GINAC_BIND_UNARCHIVER(clifford);
+GINAC_BIND_UNARCHIVER(diracone);
+GINAC_BIND_UNARCHIVER(diracgamma);
+GINAC_BIND_UNARCHIVER(diracgamma5);
+GINAC_BIND_UNARCHIVER(diracgammaL);
+GINAC_BIND_UNARCHIVER(diracgammaR);
 
 
 ex clifford::get_metric(const ex & i, const ex & j, bool symmetrised) const
@@ -716,6 +714,17 @@ ex dirac_ONE(unsigned char rl)
 	return clifford(ONE, rl);
 }
 
+static unsigned get_dim_uint(const ex& e)
+{
+	if (!is_a<idx>(e))
+		throw std::invalid_argument("get_dim_uint: argument is not an index");
+	ex dim = ex_to<idx>(e).get_dim();
+	if (!dim.info(info_flags::posint))
+		throw std::invalid_argument("get_dim_uint: dimension of index should be a positive integer");
+	unsigned d = ex_to<numeric>(dim).to_int();
+	return d;
+}
+
 ex clifford_unit(const ex & mu, const ex & metr, unsigned char rl)
 {
 	//static ex unit = (new cliffordunit)->setflag(status_flags::dynallocated);
@@ -737,10 +746,10 @@ ex clifford_unit(const ex & mu, const ex & metr, unsigned char rl)
 		//	chi((new symbol)->setflag(status_flags::dynallocated), n);
 		idx xi((new symbol)->setflag(status_flags::dynallocated), n),
 			chi((new symbol)->setflag(status_flags::dynallocated), n);
-		if ((n ==  M.cols()) && (n == ex_to<idx>(mu).get_dim())) {
+		if ((n ==  M.cols()) && (n == get_dim_uint(mu))) {
 			for (unsigned i = 0; i < n; i++) {
 				for (unsigned j = i+1; j < n; j++) {
-					if (M(i, j) != M(j, i)) {
+					if (!M(i, j).is_equal(M(j, i))) {
 						symmetric = false;
 					}
 				}
@@ -800,19 +809,11 @@ ex dirac_slash(const ex & e, const ex & dim, unsigned char rl)
    return clifford(e, varidx(0, dim), indexed((new minkmetric)->setflag(status_flags::dynallocated), symmetric2(), xi, chi), rl);
 }
 
-/** Check whether a given tinfo key (as returned by return_type_tinfo()
- *  is that of a clifford object (with an arbitrary representation label). */
-bool is_clifford_tinfo(tinfo_t ti)
-{
-	p_int start_loc=(p_int)&clifford::return_type_tinfo_static;
-	return (p_int)ti>=start_loc && (p_int)ti<start_loc+256;
-}
-
 /** Extract representation label from tinfo key (as returned by
  *  return_type_tinfo()). */
-static unsigned char get_representation_label(tinfo_t ti)
+static unsigned char get_representation_label(const return_type_t& ti)
 {
-	return (unsigned char)((p_int)ti-(p_int)&clifford::return_type_tinfo_static);
+	return (unsigned char)ti.rl;
 }
 
 /** Take trace of a string of an even number of Dirac gammas given a vector
@@ -1175,7 +1176,7 @@ ex lst_to_clifford(const ex & v, const ex & e) {
 		ex mu = e.op(1);
 		ex mu_toggle
 			= is_a<varidx>(mu) ? ex_to<varidx>(mu).toggle_variance() : mu;
-		unsigned dim = (ex_to<numeric>(ex_to<idx>(mu).get_dim())).to_int();
+		unsigned dim = get_dim_uint(mu);
 
 		if (is_a<matrix>(v)) {
 			if (ex_to<matrix>(v).cols() > ex_to<matrix>(v).rows()) {
@@ -1223,22 +1224,27 @@ static ex get_clifford_comp(const ex & e, const ex & c)
 	else if (is_a<ncmul>(e) || is_a<mul>(e)) {
 		// find a Clifford unit with the same metric, delete it and substitute its index
 		size_t ind = e.nops() + 1;
-		for (size_t j = 0; j < e.nops(); j++) 
-			if (is_a<clifford>(e.op(j)) && ex_to<clifford>(c).same_metric(e.op(j)))
-				if (ind > e.nops()) 
+		for (size_t j = 0; j < e.nops(); j++) {
+			if (is_a<clifford>(e.op(j)) && ex_to<clifford>(c).same_metric(e.op(j))) {
+				if (ind > e.nops()) {
 					ind = j;
-				else 
+				}
+				else {
 					throw(std::invalid_argument("get_clifford_comp(): expression is a Clifford multi-vector"));
+				}
+			}
+		}
 		if (ind < e.nops()) {
 			ex S = 1;
 			bool same_value_index, found_dummy;
 			same_value_index = ( ex_to<idx>(e.op(ind).op(1)).is_numeric()
 								 &&  (ival == ex_to<numeric>(ex_to<idx>(e.op(ind).op(1)).get_value()).to_int()) );
 			found_dummy = same_value_index;
-			for(size_t j=0; j < e.nops(); j++)
-				if (j != ind) 
-					if (same_value_index) 
+			for (size_t j=0; j < e.nops(); j++) {
+				if (j != ind) {
+					if (same_value_index) {
 						S = S * e.op(j);
+					}
 					else {
 						exvector ind_vec = ex_to<indexed>(e.op(j)).get_dummy_indices(ex_to<indexed>(e.op(ind)));
 						if (ind_vec.size() > 0) {
@@ -1256,6 +1262,8 @@ static ex get_clifford_comp(const ex & e, const ex & c)
 						} else
 							S = S * e.op(j);
 					}
+				}
+			}
 			return (found_dummy ? S : 0);
 		} else
 			throw(std::invalid_argument("get_clifford_comp(): expression is not a Clifford vector to the given units"));
