@@ -4,7 +4,7 @@
  *  somewhat obsolete (most of this can be replaced by exceptions). */
 
 /*
- *  GiNaC Copyright (C) 1999-2008 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2009 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -21,12 +21,12 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include <iostream>
-
 #include "fail.h"
 #include "archive.h"
 #include "utils.h"
 
+#include <iostream>
+
 namespace GiNaC {
 
 GINAC_IMPLEMENT_REGISTERED_CLASS_OPT(fail, basic,
@@ -43,13 +43,12 @@ DEFAULT_CTOR(fail)
 // archiving
 //////////
 
-DEFAULT_ARCHIVING(fail)
-
 //////////
 // functions overriding virtual functions from base classes
 //////////
 
 DEFAULT_COMPARE(fail)
 DEFAULT_PRINT(fail, "FAIL")
+GINAC_BIND_UNARCHIVER(fail);
 
 } // namespace GiNaC
