@@ -3,7 +3,7 @@
  *  Collection of all flags used through the GiNaC framework. */
 
 /*
- *  GiNaC Copyright (C) 1999-2008 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2009 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -20,8 +20,8 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#ifndef __GINAC_FLAGS_H__
-#define __GINAC_FLAGS_H__
+#ifndef GINAC_FLAGS_H
+#define GINAC_FLAGS_H
 
 namespace GiNaC {
 
@@ -281,6 +281,15 @@ public:
 	};
 };
 
+/** Flags to control the polynomial factorization. */
+class factor_options {
+public:
+	enum {
+		polynomial = 0x0000, ///< factor only expressions that are polynomials
+		all        = 0x0001  ///< factor all polynomial subexpressions
+	};
+};
+
 } // namespace GiNaC
 
-#endif // ndef __GINAC_FLAGS_H__
+#endif // ndef GINAC_FLAGS_H
