@@ -3,7 +3,7 @@
  *  Implementation of GiNaC's lst. */
 
 /*
- *  GiNaC Copyright (C) 1999-2007 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2009 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -21,6 +21,7 @@
  */
 
 #include "lst.h"
+#include "archive.h"
 
 namespace GiNaC {
 
@@ -37,4 +38,6 @@ template <> bool lst::info(unsigned inf) const
 		return inherited::info(inf);
 }
 
+GINAC_BIND_UNARCHIVER(lst);
+
 } // namespace GiNaC
