@@ -3,7 +3,7 @@
  *  Implementation of GiNaC's products of expressions. */
 
 /*
- *  GiNaC Copyright (C) 1999-2008 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2009 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -20,11 +20,6 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include <iostream>
-#include <vector>
-#include <stdexcept>
-#include <limits>
-
 #include "mul.h"
 #include "add.h"
 #include "power.h"
@@ -37,6 +32,11 @@
 #include "symbol.h"
 #include "compiler.h"
 
+#include <iostream>
+#include <limits>
+#include <stdexcept>
+#include <vector>
+
 namespace GiNaC {
 
 GINAC_IMPLEMENT_REGISTERED_CLASS_OPT(mul, expairseq,
@@ -53,7 +53,6 @@ GINAC_IMPLEMENT_REGISTERED_CLASS_OPT(mul, expairseq,
 
 mul::mul()
 {
-	tinfo_key = &mul::tinfo_static;
 }
 
 //////////
@@ -64,7 +63,6 @@ mul::mul()
 
 mul::mul(const ex & lh, const ex & rh)
 {
-	tinfo_key = &mul::tinfo_static;
 	overall_coeff = _ex1;
 	construct_from_2_ex(lh,rh);
 	GINAC_ASSERT(is_canonical());
@@ -72,7 +70,6 @@ mul::mul(const ex & lh, const ex & rh)
 
 mul::mul(const exvector & v)
 {
-	tinfo_key = &mul::tinfo_static;
 	overall_coeff = _ex1;
 	construct_from_exvector(v);
 	GINAC_ASSERT(is_canonical());
@@ -80,7 +77,6 @@ mul::mul(const exvector & v)
 
 mul::mul(const epvector & v)
 {
-	tinfo_key = &mul::tinfo_static;
 	overall_coeff = _ex1;
 	construct_from_epvector(v);
 	GINAC_ASSERT(is_canonical());
@@ -88,7 +84,6 @@ mul::mul(const epvector & v)
 
 mul::mul(const epvector & v, const ex & oc, bool do_index_renaming)
 {
-	tinfo_key = &mul::tinfo_static;
 	overall_coeff = oc;
 	construct_from_epvector(v, do_index_renaming);
 	GINAC_ASSERT(is_canonical());
@@ -96,7 +91,6 @@ mul::mul(const epvector & v, const ex & oc, bool do_index_renaming)
 
 mul::mul(std::auto_ptr<epvector> vp, const ex & oc, bool do_index_renaming)
 {
-	tinfo_key = &mul::tinfo_static;
 	GINAC_ASSERT(vp.get()!=0);
 	overall_coeff = oc;
 	construct_from_epvector(*vp, do_index_renaming);
@@ -105,7 +99,6 @@ mul::mul(std::auto_ptr<epvector> vp, const ex & oc, bool do_index_renaming)
 
 mul::mul(const ex & lh, const ex & mh, const ex & rh)
 {
-	tinfo_key = &mul::tinfo_static;
 	exvector factors;
 	factors.reserve(3);
 	factors.push_back(lh);
@@ -120,8 +113,6 @@ mul::mul(const ex & lh, const ex & mh, const ex & rh)
 // archiving
 //////////
 
-DEFAULT_ARCHIVING(mul)
-
 //////////
 // functions overriding virtual functions from base classes
 //////////
@@ -637,7 +628,7 @@ ex mul::eval_ncmul(const exvector & v) const
 	return inherited::eval_ncmul(v);
 }
 
-bool tryfactsubs(const ex & origfactor, const ex & patternfactor, int & nummatches, lst & repls)
+bool tryfactsubs(const ex & origfactor, const ex & patternfactor, int & nummatches, exmap& repls)
 {	
 	ex origbase;
 	int origexponent;
@@ -669,7 +660,7 @@ bool tryfactsubs(const ex & origfactor, const ex & patternfactor, int & nummatch
 		patternexpsign = 1;
 	}
 
-	lst saverepls = repls;
+	exmap saverepls = repls;
 	if (origexponent < patternexponent || origexpsign != patternexpsign || !origbase.match(patternbase,saverepls))
 		return false;
 	repls = saverepls;
@@ -688,17 +679,17 @@ bool tryfactsubs(const ex & origfactor, const ex & patternfactor, int & nummatch
   * that already have been replaced by previous substitutions and matched[i]
   * is true for factors that have been matched by the current match.
   */
-bool algebraic_match_mul_with_mul(const mul &e, const ex &pat, lst &repls,
+bool algebraic_match_mul_with_mul(const mul &e, const ex &pat, exmap& repls,
 		int factor, int &nummatches, const std::vector<bool> &subsed,
 		std::vector<bool> &matched)
 {
-	if (factor == pat.nops())
+	if (factor == (int)pat.nops())
 		return true;
 
 	for (size_t i=0; i<e.nops(); ++i) {
 		if(subsed[i] || matched[i])
 			continue;
-		lst newrepls = repls;
+		exmap newrepls = repls;
 		int newnummatches = nummatches;
 		if (tryfactsubs(e.op(i), pat.op(factor), newnummatches, newrepls)) {
 			matched[i] = true;
@@ -721,7 +712,7 @@ bool mul::has(const ex & pattern, unsigned options) const
 	if(!(options&has_options::algebraic))
 		return basic::has(pattern,options);
 	if(is_a<mul>(pattern)) {
-		lst repls;
+		exmap repls;
 		int nummatches = std::numeric_limits<int>::max();
 		std::vector<bool> subsed(seq.size(), false);
 		std::vector<bool> matched(seq.size(), false);
@@ -745,7 +736,7 @@ ex mul::algebraic_subs_mul(const exmap & m, unsigned options) const
 retry1:
 			int nummatches = std::numeric_limits<int>::max();
 			std::vector<bool> currsubsed(seq.size(), false);
-			lst repls;
+			exmap repls;
 			
 			if(!algebraic_match_mul_with_mul(*this, it->first, repls, 0, nummatches, subsed, currsubsed))
 				continue;
@@ -754,10 +745,10 @@ retry1:
 				if (currsubsed[j])
 					subsed[j] = true;
 			ex subsed_pattern
-				= it->first.subs(ex(repls), subs_options::no_pattern);
+				= it->first.subs(repls, subs_options::no_pattern);
 			divide_by *= power(subsed_pattern, nummatches);
 			ex subsed_result
-				= it->second.subs(ex(repls), subs_options::no_pattern);
+				= it->second.subs(repls, subs_options::no_pattern);
 			multiply_by *= power(subsed_result, nummatches);
 			goto retry1;
 
@@ -765,14 +756,14 @@ retry1:
 
 			for (size_t j=0; j<this->nops(); j++) {
 				int nummatches = std::numeric_limits<int>::max();
-				lst repls;
+				exmap repls;
 				if (!subsed[j] && tryfactsubs(op(j), it->first, nummatches, repls)){
 					subsed[j] = true;
 					ex subsed_pattern
-						= it->first.subs(ex(repls), subs_options::no_pattern);
+						= it->first.subs(repls, subs_options::no_pattern);
 					divide_by *= power(subsed_pattern, nummatches);
 					ex subsed_result
-						= it->second.subs(ex(repls), subs_options::no_pattern);
+						= it->second.subs(repls, subs_options::no_pattern);
 					multiply_by *= power(subsed_result, nummatches);
 				}
 			}
@@ -855,10 +846,10 @@ unsigned mul::return_type() const
 	return all_commutative ? return_types::commutative : return_types::noncommutative;
 }
    
-tinfo_t mul::return_type_tinfo() const
+return_type_t mul::return_type_tinfo() const
 {
 	if (seq.empty())
-		return this;  // mul without factors: should not happen
+		return make_return_type_t<mul>(); // mul without factors: should not happen
 	
 	// return type_info of first noncommutative element
 	epvector::const_iterator i = seq.begin(), end = seq.end();
@@ -868,7 +859,7 @@ tinfo_t mul::return_type_tinfo() const
 		++i;
 	}
 	// no noncommutative element found, should not happen
-	return this;
+	return make_return_type_t<mul>();
 }
 
 ex mul::thisexpairseq(const epvector & v, const ex & oc, bool do_index_renaming) const
@@ -1217,4 +1208,6 @@ std::auto_ptr<epvector> mul::expandchildren(unsigned options) const
 	return std::auto_ptr<epvector>(0); // nothing has changed
 }
 
+GINAC_BIND_UNARCHIVER(mul);
+
 } // namespace GiNaC
