@@ -4,7 +4,7 @@
  *  of any interest to the user of the library. */
 
 /*
- *  GiNaC Copyright (C) 1999-2008 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2009 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -21,20 +21,19 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#ifndef __GINAC_UTILS_H__
-#define __GINAC_UTILS_H__
+#ifndef GINAC_UTILS_H
+#define GINAC_UTILS_H
 
+#include "assertion.h"
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
-#include <string>
 #include <functional>
 #ifdef HAVE_STDINT_H
 #include <stdint.h> // for uintptr_t
 #endif
-
-#include "assertion.h"
+#include <string>
 
 namespace GiNaC {
 
@@ -49,6 +48,13 @@ class dunno {};
 
 unsigned log2(unsigned n);
 
+/** Rotate bits of unsigned value by one bit to the left.
+  * This can be necesary if the user wants to define its own hashes. */
+inline unsigned rotate_left(unsigned n)
+{
+	return (n & 0x80000000U) ? (n << 1 | 0x00000001U) : (n << 1);
+}
+
 /** Compare two pointers (just to establish some sort of canonical order).
  *  @return -1, 0, or 1 */
 template <class T>
@@ -398,21 +404,7 @@ extern const ex _ex120;
 // Helper macros for class implementations (mostly useful for trivial classes)
 
 #define DEFAULT_CTOR(classname) \
-classname::classname() : inherited(&classname::tinfo_static) { setflag(status_flags::evaluated | status_flags::expanded); }
-
-#define DEFAULT_UNARCHIVE(classname) \
-ex classname::unarchive(const archive_node &n, lst &sym_lst) \
-{ \
-	return (new classname(n, sym_lst))->setflag(status_flags::dynallocated); \
-}
-
-#define DEFAULT_ARCHIVING(classname) \
-classname::classname(const archive_node &n, lst &sym_lst) : inherited(n, sym_lst) {} \
-DEFAULT_UNARCHIVE(classname) \
-void classname::archive(archive_node &n) const \
-{ \
-	inherited::archive(n); \
-}
+classname::classname() { setflag(status_flags::evaluated | status_flags::expanded); }
 
 #define DEFAULT_COMPARE(classname) \
 int classname::compare_same_type(const basic & other) const \
@@ -436,5 +428,4 @@ void classname::do_print_latex(const print_latex & c, unsigned level) const \
 
 } // namespace GiNaC
 
-
-#endif // ndef __GINAC_UTILS_H__
+#endif // ndef GINAC_UTILS_H
